import booleanOperations
try:
    from mojo.UI import getDefault, setDefault
    hasMojo = True
except ImportError:
    hasMojo = False

try:
    CurrentFont
except NameError:
    class CurrentFont(dict):

        def save(self, path=None):
            pass


f = CurrentFont()

if hasMojo:
    glyphViewRoundValues = getDefault("glyphViewRoundValues")
    setDefault("glyphViewRoundValues", 0)


for g in f:
    g.leftMargin = 0
    g.rightMargin = 0
    n = g.naked()
    d = g.getLayer("union")
    d.clear()
    d.appendGlyph(g)
    d.removeOverlap(round=0)

    if len(g) > 1:
        for method in "xor", "difference", "intersection":
            d = g.getLayer(method)
            d.clear()
            func = getattr(booleanOperations, method)
            func([n[0]], n[1:], d.getPointPen())

f.save()

if hasMojo:
    setDefault("glyphViewRoundValues", glyphViewRoundValues)


from .booleanOperationManager import BooleanOperationManager
from .exceptions import BooleanOperationsError

try:
    from ._version import version as __version__
except ImportError:
    __version__ = "0.0.0+unknown"

# export BooleanOperationManager static methods
union = BooleanOperationManager.union
difference = BooleanOperationManager.difference
intersection = BooleanOperationManager.intersection
xor = BooleanOperationManager.xor
getIntersections = BooleanOperationManager.getIntersections



# run in DrawBot RoboFont extension
border = 20
dotSize = 10
offDotSize = dotSize * .5

try:
    CurrentFont
except NameError:
    class CurrentFont(dict):

        glyphOrder = []

        def save(self, path=None):
            pass

try:
    saveImage
except NameError:
    def saveImage(*args, **kwargs):
        pass


f = CurrentFont()


def drawOffCurve(anchor, off):
    x, y = anchor
    offx, offy = off
    if offx or offy:
        offx += x
        offy += y
        with savedState():
            stroke(1, 0, 0)
            fill(1, 0, 0)
            line((x, y), (offx, offy))
            oval(offx - offDotSize, offy - offDotSize, offDotSize * 2, offDotSize * 2)


def drawGlyphWithPoints(glyph):
    fill(0, .1)
    stroke(0)
    drawGlyph(glyph)
    stroke(None)

    for contour in glyph:
        fill(0, 1, 0)
        for point in contour.bPoints:
            x, y = point.anchor
            drawOffCurve((x, y), point.bcpIn)
            drawOffCurve((x, y), point.bcpOut)
            oval(x - dotSize, y - dotSize, dotSize * 2, dotSize * 2)
            fill(1, 0, 0)


for glyphName in f.glyphOrder:
    if glyphName not in f:
        continue
    g = f[glyphName]
    bounds = g.bounds
    if not bounds:
        continue
    minx, miny, maxx, maxy = bounds
    w = maxx - minx
    h = maxy - miny
    layerCount = len(f.layers)
    newPage((w + border) * layerCount + border, h  + border * 2 + 100)
    translate(border, border + 100)
    translate(-minx, -miny)
    fontSize(20)
    stroke()
    text("%s" % g.name, (w * .5, -100 + miny), align="center")
    drawGlyphWithPoints(g)
    translate(w + border, 0)
    for layer in f.layers:
        if layer.name == "foreground":
            continue
        fill(0)
        text(layer.name, (w * .5, -100 + miny), align="center")
        if g.name not in layer:
            translate(w + border)
            continue
        lg = layer[g.name]
        drawGlyphWithPoints(lg)
        translate(w + border)


saveImage("visualTest.pdf")


from __future__ import print_function, division, absolute_import

import sys
import os
import unittest

import pytest
from fontPens.digestPointPen import DigestPointPen

import defcon
import booleanOperations


VERBOSE = False


class BooleanTests(unittest.TestCase):

    pass


def _makeTestCase(glyph, booleanMethodName, args=None):
    # get the font
    font = glyph.font
    # skip if the booleanMethodName does not exist as layer
    if booleanMethodName not in font.layers:
        return False, None
    expectedLayer = font.layers[booleanMethodName]
    # skip if the glyph name does not exist in the expected layer
    if glyph.name not in expectedLayer:
        return False, None
    expectedGlyph = expectedLayer[glyph.name]

    if args is None:
        if len(glyph) < 2:
            # skip if not args are given and the glyph has only 1 contour
            return False, None
        # set the first contour as subject contour and the rest as clip contour
        args = [[glyph[0]], glyph[1:]]

    func = getattr(booleanOperations, booleanMethodName)

    def test(self):
        if VERBOSE:
            print("test: '%s' for '%s'" % (glyph.name, booleanMethodName))
        testPen = DigestPointPen()
        func(*args, outPen=testPen)
        expectedPen = DigestPointPen()
        expectedGlyph.drawPoints(expectedPen)
        self.assertEqual(testPen.getDigest(), expectedPen.getDigest(), "Glyph name '%s' failed for '%s'." % (glyph.name, booleanMethodName))

    return True, test


def _makeUnionTestCase(glyph, method):
    return _makeTestCase(glyph, method, args=[glyph])


def _addGlyphTests():
    root = os.path.join(os.path.dirname(__file__), 'testData')
    path = os.path.join(root, "test.ufo")
    font = defcon.Font(path)

    booleanMethods = {
        "union": _makeUnionTestCase,
        "difference": _makeTestCase,
        "intersection": _makeTestCase,
        "xor": _makeTestCase,
    }

    for glyph in font:
        for booleanMethod, testMaker in booleanMethods.items():
            shouldPerformTest, testMethod = testMaker(glyph, booleanMethod)
            if shouldPerformTest:
                testMethodName = "test_%s_%s" % (glyph.name, booleanMethod)
                testMethod.__name__ = str(testMethodName)
                setattr(BooleanTests, testMethodName, testMethod)


_addGlyphTests()

def test_unsupported_qcurve():
    font = defcon.Font()
    g = font.newGlyph("test")
    p = g.getPointPen()
    p.beginPath()
    p.addPoint((0, 0), segmentType="line")
    p.addPoint((100, 0), segmentType="line")
    p.addPoint((100, 100), segmentType="line")
    p.addPoint((50, 100))
    p.addPoint((0, 100), segmentType="qcurve")
    p.endPath()

    with pytest.raises(booleanOperations.exceptions.UnsupportedContourError):
        booleanOperations.union(g, None)

if __name__ == '__main__':
    sys.exit(unittest.main())
